@@ -10,6 +10,7 @@
 #include "tangle-actor.h"
 #include "tangle-action.h"
 #include "tangle-properties.h"
+#include "tangle-vault.h"
 #include "tangle-misc.h"
 #include "marshalers.h"
 #include <gobject/gvaluecollector.h>
@@ -57,6 +58,7 @@ struct _TangleActorPrivate {
 	TangleAction* leave_action;
 	TangleAction* motion_action;
 	TangleAction* scroll_action;
+	TangleAction* transition_completed_action;
 	
 	GList* ancestor_events;
 	GList* descendant_events;
@@ -105,14 +107,17 @@ enum {
 	PROP_ENTER_ACTION,
 	PROP_LEAVE_ACTION,
 	PROP_MOTION_ACTION,
-	PROP_SCROLL_ACTION
+	PROP_SCROLL_ACTION,
+	PROP_TRANSITION_COMPLETED_ACTION
 };
 
 enum {
 	ANIMATE_TRANSITION,
+	TRANSITION_COMPLETED,
 	LAST_SIGNAL
 };
 
+static ClutterAnimation* animate_transition(TangleActor* actor, ClutterActorBox* from_allocation, ClutterActorBox* to_allocation);
 static void on_animation_completed(ClutterAnimation* animation, gpointer user_data);
 static void set_max_width(TangleActor* actor, gfloat max_width);
 static void set_max_height(TangleActor* actor, gfloat max_height);
@@ -193,8 +198,7 @@ void tangle_actor_show(TangleActor* actor) {
 
 		g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 		actor->priv->transition_scale_children = TRUE;
-		g_signal_emit(actor, signals[ANIMATE_TRANSITION], 0, NULL, current_allocation, &animation);
-		if (animation) {
+		if ((animation = animate_transition(actor, NULL, current_allocation))) {
 			g_signal_connect_swapped(animation, "completed", G_CALLBACK(unset_transition_scale_children), actor);
 		}
 		
@@ -216,7 +220,7 @@ void tangle_actor_hide_animated(TangleActor* actor) {
 		if (TANGLE_ACTOR_IS_PAINTED(actor)) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
-			g_signal_emit(actor, signals[ANIMATE_TRANSITION], 0, current_allocation, NULL, &animation);
+			animation = animate_transition(actor, current_allocation, NULL);
 		}
 		if (animation) {
 			actor->priv->hiding_animation = animation;
@@ -242,7 +246,7 @@ void tangle_actor_hide_all_animated(TangleActor* actor) {
 		if (TANGLE_ACTOR_IS_PAINTED(actor)) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
-			g_signal_emit(actor, signals[ANIMATE_TRANSITION], 0, current_allocation, NULL, &animation);
+			animation = animate_transition(actor, current_allocation, NULL);
 		}
 		if (animation) {
 			actor->priv->hiding_animation = animation;
@@ -270,7 +274,7 @@ void tangle_actor_destroy_animated(TangleActor* actor) {
 		if (TANGLE_ACTOR_IS_PAINTED(actor)) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
-			g_signal_emit(actor, signals[ANIMATE_TRANSITION], 0, current_allocation, NULL, &animation);
+			animation = animate_transition(actor, current_allocation, NULL);
 		}
 		if (animation) {
 			actor->priv->hiding_animation = animation;
@@ -775,7 +779,7 @@ static void tangle_actor_allocate(ClutterActor* self, const ClutterActorBox* box
 		if (!old_allocation) {
 			actor->priv->transition_scale_children = TRUE;
 		}
-		g_signal_emit(self, signals[ANIMATE_TRANSITION], 0, old_allocation, new_allocation, &animation);
+		animation = animate_transition(actor, old_allocation, new_allocation);
 		if (animation && !old_allocation) {
 			g_signal_connect_swapped(animation, "completed", G_CALLBACK(unset_transition_scale_children), self);
 		}
@@ -950,6 +954,18 @@ static ClutterAnimation* tangle_actor_animate_transition(TangleActor* actor, Clu
 	return animation;
 }
 
+static void tangle_actor_transition_completed(TangleActor* actor, ClutterActorBox* old_box, ClutterActorBox* current_box) {
+	TangleProperties* properties;
+	
+	if (actor->priv->transition_completed_action) {
+		properties = tangle_properties_new();
+		tangle_properties_set(properties, "old-allocation", CLUTTER_TYPE_ACTOR_BOX, old_box, "current-allocation", CLUTTER_TYPE_ACTOR_BOX, current_box, NULL);
+		tangle_action_execute_full(actor->priv->transition_completed_action, G_OBJECT(actor), "transition-completed", properties);
+		g_object_unref(G_OBJECT(properties));
+	}
+	
+}
+
 static void tangle_actor_paint_event_handler(ClutterActor* self, gpointer user_data) {
 	TangleActor* actor;
 
@@ -1065,6 +1081,9 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 		case PROP_SCROLL_ACTION:
 			SET_ACTION(scroll_action);
 			break;
+		case PROP_TRANSITION_COMPLETED_ACTION:
+			SET_ACTION(transition_completed_action);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -1136,6 +1155,9 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
 		case PROP_SCROLL_ACTION:
 			g_value_set_object(value, actor->priv->scroll_action);
 			break;
+		case PROP_TRANSITION_COMPLETED_ACTION:
+			g_value_set_object(value, actor->priv->transition_completed_action);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -1182,6 +1204,7 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	clutter_actor_class->scroll_event = tangle_actor_scroll_event;
 	
 	actor_class->animate_transition = tangle_actor_animate_transition;
+	actor_class->transition_completed = tangle_actor_transition_completed;
 	actor_class->handle_ancestor_event = tangle_actor_real_handle_ancestor_event;
 	actor_class->handle_descendant_event = tangle_actor_real_handle_descendant_event;
 
@@ -1365,6 +1388,15 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	                                "The action that correspond to the scroll event",
 	                                TANGLE_TYPE_ACTION,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:animation-completed-action:
+	 */
+	g_object_class_install_property(gobject_class, PROP_TRANSITION_COMPLETED_ACTION,
+	                                g_param_spec_object("transition-completed-action",
+	                                "Transition completed action",
+	                                "The action that correspond to the transition completed signal",
+	                                TANGLE_TYPE_ACTION,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 
 	/**
@@ -1380,6 +1412,19 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 						   tangle_marshal_OBJECT__BOXED_BOXED,
 						   CLUTTER_TYPE_ANIMATION, 2,
 						   CLUTTER_TYPE_ACTOR_BOX, CLUTTER_TYPE_ACTOR_BOX);
+	/**
+	 * TangleActor::transition-completed:
+	 * @actor: the object which received the signal
+	 *
+	 * The ::transition-completed signal is emitted when the transition animation (see ::animate-transition)
+	 * is completed, or immediately if there is no animation.
+	 */
+	signals[TRANSITION_COMPLETED] = g_signal_new("transition-completed", G_TYPE_FROM_CLASS(gobject_class),
+	                                             G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET(TangleActorClass, transition_completed),
+						     NULL, NULL,
+						     tangle_marshal_VOID__BOXED_BOXED,
+						     G_TYPE_NONE, 2,
+						     CLUTTER_TYPE_ACTOR_BOX, CLUTTER_TYPE_ACTOR_BOX);
 
 	g_type_class_add_private (gobject_class, sizeof (TangleActorPrivate));
 }
@@ -1398,6 +1443,32 @@ static void tangle_actor_init(TangleActor* actor) {
 	actor->priv->pick_event_handler_id = g_signal_connect(actor, "pick", G_CALLBACK(tangle_actor_pick_event_handler), NULL);
 }
 
+static void on_transition_animation_completed(ClutterAnimation* animation, gpointer user_data) {
+	TangleVault* vault;
+	TangleActor* actor;
+	ClutterActorBox* from_allocation;
+	ClutterActorBox* to_allocation;
+	
+	vault = TANGLE_VAULT(user_data);
+	tangle_vault_get(vault, 3, TANGLE_TYPE_ACTOR, &actor, CLUTTER_TYPE_ACTOR_BOX, &from_allocation, CLUTTER_TYPE_ACTOR_BOX, &to_allocation);
+	g_signal_emit(actor, signals[TRANSITION_COMPLETED], 0, from_allocation, to_allocation);
+}
+
+static ClutterAnimation* animate_transition(TangleActor* actor, ClutterActorBox* from_allocation, ClutterActorBox* to_allocation) {
+	ClutterAnimation* animation = NULL;
+	TangleVault* vault;
+	
+	g_signal_emit(actor, signals[ANIMATE_TRANSITION], 0, from_allocation, to_allocation, &animation);
+	if (animation) {
+		vault = tangle_vault_new(3, TANGLE_TYPE_ACTOR, actor, CLUTTER_TYPE_ACTOR_BOX, from_allocation, CLUTTER_TYPE_ACTOR_BOX, to_allocation);
+		tangle_signal_connect_vault(animation, "completed", G_CALLBACK(on_transition_animation_completed), vault);
+	} else {
+		g_signal_emit(actor, signals[TRANSITION_COMPLETED], 0, from_allocation, to_allocation);
+	}
+
+	return animation;
+}
+
 static void set_coordinates_by_gravity(ClutterGravity gravity, gfloat width, gfloat height, gfloat* x_p, gfloat* y_p) {
 	switch (gravity) {
 		case CLUTTER_GRAVITY_NORTH_WEST:
