@@ -120,10 +120,11 @@ void tangle_style_apply(TangleStyle* style, GObject* object) {
 			tangle_style_foreach_style_property(old_style, unset_style_property, object);
 		}
 		hash_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
-		g_object_set_qdata_full(object, quark_object_style_properties, hash_table,(GDestroyNotify)g_hash_table_unref);
+		g_object_set_qdata_full(object, quark_object_style_properties, hash_table, (GDestroyNotify)g_hash_table_unref);
 		g_signal_connect(object, "notify", G_CALLBACK(on_notify), hash_table);
 
 		tangle_style_foreach_style_property(style, set_style_property, object);
+		g_object_ref(style);
 		g_object_set_qdata(object, quark_object_style, style);
 				
 		g_signal_connect(style, "style-changed", G_CALLBACK(on_style_changed), object);
@@ -138,6 +139,7 @@ void tangle_style_unapply(TangleStyle* style, GObject* object) {
 	old_style = TANGLE_STYLE(g_object_get_qdata(object, quark_object_style));
 	if (old_style == style) {
 		tangle_style_foreach_style_property(style, unset_style_property, object);
+		g_object_unref(style);
 		g_object_set_qdata(object, quark_object_style, NULL);
 		
 		hash_table = (GHashTable*)g_object_get_qdata(object, quark_object_style_properties);
