@@ -747,7 +747,9 @@ static void tangle_widget_add_impl(TangleWidget* widget, ClutterActor* actor, Cl
 	if (widget->priv->stylesheet) {
 		tangle_stylesheet_apply(widget->priv->stylesheet, G_OBJECT(actor));
 	}
-	g_signal_emit_by_name(widget, "actor-added", actor);
+
+	/* TODO: g_signal_emit_by_name(widget, "actor-added", actor); */
+
 	clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
 }
 
@@ -769,7 +771,9 @@ static void tangle_widget_remove(ClutterContainer* widget, ClutterActor* actor)
 		tangle_stylesheet_unapply(TANGLE_WIDGET(widget)->priv->stylesheet, G_OBJECT(actor));
 	}
 	clutter_actor_unparent(actor);
-	g_signal_emit_by_name(widget, "actor-removed", actor);
+	
+	/* TODO: g_signal_emit_by_name(widget, "actor-removed", actor); */
+
 	if (was_visible) {
 		clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
 	}
